;;;                 -*- Mode: Lisp; Package: BOOMS-APP -*-                  ;;;
;;; ----------------------------------------------------------------------- ;;;
;;; File:         coledit.lsp                                               ;;;
;;; Description:  BOOMS icon color edit page                                ;;;
;;; Author:       Eli Barzilay - eli@cs.bgu.ac.il                           ;;;
;;; Date:         9-10-96                                                   ;;;
;;; Package:      BOOMS-APP                                                 ;;;
;;; ----------------------------------------------------------------------- ;;;

#|

This file is part of the BOOMS system:
+-----------------------------------------------------------------------------+
|                                                                             |
| /######   /#####   /#####  /##     /##  /#####                              |
| /##///## /##///## /##///## /###   /### /##///##                        /--- |
| /##  /## /##  /## /##  /## /#### /#### /##  ///          /------/     /     |
| /######/ /##  /## /##  /## /##/####/## //#####          /      /     /      |
| /##///## /##  /## /##  /## /##//##//##  /////##        /------/ __  /       |
| /##  /## /##  /## /##  /## /## /// /##      /##   __  /  __  / /  \/        |
| /##  /## /##  /## /##  /## /##     /## /##  /##  /  \/  /  \/  \__/         |
| /######/ //#####/ //#####/ /##     /## //#####/  \__/   \__/                |
| ///////   //////   //////  ///     ///  //////                              |
|                                                                             |
|                   Booms Object Oriented Music Structures.                   |
|Eli Barzilay:                                                   Maze is Life!|
+-----------------------------------------------------------------------------+

|#


;;;-------------------------------------------------------------------------;;;
;;;  C O L E D I T  file                                                    ;;;
;;;-------------------------------------------------------------------------;;;

(in-package :booms-app)


;;;-------------------------------------------------------------------------;;;
;;;  C O L O R S  variables                                                 ;;;
;;;-------------------------------------------------------------------------;;;

;;; Foreground variables.
(defvar *COLOR-FORE-RED*   nil)
(defvar *COLOR-FORE-GREEN* nil)
(defvar *COLOR-FORE-BLUE*  nil)
(defvar *COLOR-BOX-FORE*   nil)

;;; Background variables.
(defvar *COLOR-BACK-RED*   nil)
(defvar *COLOR-BACK-GREEN* nil)
(defvar *COLOR-BACK-BLUE*  nil)
(defvar *COLOR-BOX-BACK*   nil)

;;; Selection variables.
(defvar *COLOR-SEL-RED*    nil)
(defvar *COLOR-SEL-GREEN*  nil)
(defvar *COLOR-SEL-BLUE*   nil)
(defvar *COLOR-BOX-SEL*    nil)


;;;-------------------------------------------------------------------------;;;
;;;  C O L O R S  initializations                                           ;;;
;;;-------------------------------------------------------------------------;;;

;;; This tab can be used with all icons.
(defmethod ICON-TAB-IDS ((x icon))
  (list* :colors (call-next-method)))

;;; Initialization.
(defun INITIALIZE-COLOR-EDIT ()
  (make-color-tab))

;;; Build the tab page.
(defun MAKE-COLOR-TAB ()
  (add-icon-edit-tab :colors)
  (progrev
   ;; Foreground row.
   (add-icon-edit-tab-widget
     :widget          'static-text
     :value           "~Border"
     :box             (make-box 10 27 55 52)
     :tabstop         nil
     :groupstart      t)
   (setf *color-box-fore*
    (add-icon-edit-tab-widget
     :widget         'color-box
     :value          nil
     :box            (make-box 55 27 82 52)
     :tabstop        nil
     :groupstart     nil
     :set-value-fn   #'color-box-fore-set))
   (setf *color-fore-red*
    (add-icon-edit-tab-widget
     :widget          'lisp-text
     :name            :red-fg-color-text
     :value           nil
     :box             (make-box 85 27 120 52)
     :tabstop         t
     :groupstart      nil
     :up-down-control :red-fg-color-up-down
     :set-value-fn    #'color-fore-red-set))
   (add-icon-edit-tab-widget
     :widget          'up-down-control
     :name            :red-fg-color-up-down
     :value           nil
     :box             (make-box 120 27 130 52)
     :tabstop         nil
     :groupstart      nil
     :buddy-widget    :red-fg-color-text
     :buddy-side      :right
     :delayed         nil
     :orientation     :vertical
     :wrapping        nil
     :use-arrow-keys  t
     :increment       nil
     :range           (list 0 255))
   (setf *color-fore-green*
    (add-icon-edit-tab-widget
     :widget          'lisp-text
     :name            :green-fg-color-text
     :value           nil
     :box             (make-box 135 27 170 52)
     :tabstop         t
     :groupstart      nil
     :up-down-control :green-fg-color-up-down
     :set-value-fn    #'color-fore-green-set))
   (add-icon-edit-tab-widget
     :widget          'up-down-control
     :name            :green-fg-color-up-down
     :value           nil
     :box             (make-box 170 27 180 52)
     :tabstop         nil
     :groupstart      nil
     :buddy-widget    :green-fg-color-text
     :buddy-side      :right
     :delayed         nil
     :orientation     :vertical
     :wrapping        nil
     :use-arrow-keys  t
     :increment       nil
     :range           (list 0 255))
   (setf *color-fore-blue*
    (add-icon-edit-tab-widget
     :widget          'lisp-text
     :name            :blue-fg-color-text
     :value           nil
     :box             (make-box 185 27 220 52)
     :tabstop         t
     :groupstart      nil
     :up-down-control :blue-fg-color-up-down
     :set-value-fn    #'color-fore-blue-set))
   (add-icon-edit-tab-widget
     :widget          'up-down-control
     :name            :blue-fg-color-up-down
     :value           nil
     :box             (make-box 220 27 230 52)
     :tabstop         nil
     :groupstart      nil
     :buddy-widget    :blue-fg-color-text
     :buddy-side      :right
     :delayed         nil
     :orientation     :vertical
     :wrapping        nil
     :use-arrow-keys  t
     :increment       nil
     :range           (list 0 255))
   ;; Background row.
   (add-icon-edit-tab-widget
     :widget          'static-text
     :value           "Bac~k"
     :box             (make-box 10 60 55 85)
     :tabstop         nil
     :groupstart      t)
   (setf *color-box-back*
    (add-icon-edit-tab-widget
     :widget         'color-box
     :value          nil
     :box            (make-box 55 60 82 85)
     :tabstop        nil
     :groupstart     nil
     :set-value-fn   #'color-box-back-set))
   (setf *color-back-red*
    (add-icon-edit-tab-widget
     :widget          'lisp-text
     :name            :red-bg-color-text
     :value           nil
     :box             (make-box 85 60 120 85)
     :tabstop         t
     :groupstart      nil
     :up-down-control :red-bg-color-up-down
     :set-value-fn    #'color-back-red-set))
   (add-icon-edit-tab-widget
     :widget          'up-down-control
     :name            :red-bg-color-up-down
     :value           nil
     :box             (make-box 120 60 130 85)
     :tabstop         nil
     :groupstart      nil
     :buddy-widget    :red-bg-color-text
     :buddy-side      :right
     :delayed         nil
     :orientation     :vertical
     :wrapping        nil
     :use-arrow-keys  t
     :increment       nil
     :range           (list 0 255))
   (setf *color-back-green*
    (add-icon-edit-tab-widget
     :widget          'lisp-text
     :name            :green-bg-color-text
     :value           nil
     :box             (make-box 135 60 170 85)
     :tabstop         t
     :groupstart      nil
     :up-down-control :green-bg-color-up-down
     :set-value-fn    #'color-back-green-set))
   (add-icon-edit-tab-widget
     :widget          'up-down-control
     :name            :green-bg-color-up-down
     :value           nil
     :box             (make-box 170 60 180 85)
     :tabstop         nil
     :groupstart      nil
     :buddy-widget    :green-bg-color-text
     :buddy-side      :right
     :delayed         nil
     :orientation     :vertical
     :wrapping        nil
     :use-arrow-keys  t
     :increment       nil
     :range           (list 0 255))
   (setf *color-back-blue*
    (add-icon-edit-tab-widget
     :widget          'lisp-text
     :name            :blue-bg-color-text
     :value           nil
     :box             (make-box 185 60 220 85)
     :tabstop         t
     :groupstart      nil
     :up-down-control :blue-bg-color-up-down
     :set-value-fn    #'color-back-blue-set))
   (add-icon-edit-tab-widget
     :widget          'up-down-control
     :name            :blue-bg-color-up-down
     :value           nil
     :box             (make-box 220 60 230 85)
     :tabstop         nil
     :groupstart      nil
     :buddy-widget    :blue-bg-color-text
     :buddy-side      :right
     :delayed         nil
     :orientation     :vertical
     :wrapping        nil
     :use-arrow-keys  t
     :increment       nil
     :range           (list 0 255))
   ;; Selection row.
   (add-icon-edit-tab-widget
     :widget          'static-text
     :value           "~Select"
     :box             (make-box 10 93 55 118)
     :tabstop         nil
     :groupstart      t)
   (setf *color-box-sel*
    (add-icon-edit-tab-widget
     :widget         'color-box
     :value          nil
     :box            (make-box 55 93 82 118)
     :tabstop        nil
     :groupstart     nil
     :set-value-fn   #'color-box-sel-set))
   (setf *color-sel-red*
    (add-icon-edit-tab-widget
     :widget          'lisp-text
     :name            :red-sel-color-text
     :value           nil
     :box             (make-box 85 93 120 118)
     :tabstop         t
     :groupstart      nil
     :up-down-control :red-sel-color-up-down
     :set-value-fn    #'color-sel-red-set))
   (add-icon-edit-tab-widget
     :widget          'up-down-control
     :name            :red-sel-color-up-down
     :value           nil
     :box             (make-box 120 93 130 118)
     :tabstop         nil
     :groupstart      nil
     :buddy-widget    :red-sel-color-text
     :buddy-side      :right
     :delayed         nil
     :orientation     :vertical
     :wrapping        nil
     :use-arrow-keys  t
     :increment       nil
     :range           (list 0 255))
   (setf *color-sel-green*
    (add-icon-edit-tab-widget
     :widget          'lisp-text
     :name            :green-sel-color-text
     :value           nil
     :box             (make-box 135 93 170 118)
     :tabstop         t
     :groupstart      nil
     :up-down-control :green-sel-color-up-down
     :set-value-fn    #'color-sel-green-set))
   (add-icon-edit-tab-widget
     :widget          'up-down-control
     :name            :green-sel-color-up-down
     :value           nil
     :box             (make-box 170 93 180 118)
     :tabstop         nil
     :groupstart      nil
     :buddy-widget    :green-sel-color-text
     :buddy-side      :right
     :delayed         nil
     :orientation     :vertical
     :wrapping        nil
     :use-arrow-keys  t
     :increment       nil
     :range           (list 0 255))
   (setf *color-sel-blue*
    (add-icon-edit-tab-widget
     :widget          'lisp-text
     :name            :blue-sel-color-text
     :value           nil
     :box             (make-box 185 93 220 118)
     :tabstop         t
     :groupstart      nil
     :up-down-control :blue-sel-color-up-down
     :set-value-fn    #'color-sel-blue-set))
   (add-icon-edit-tab-widget
     :widget          'up-down-control
     :name            :blue-sel-color-up-down
     :value           nil
     :box             (make-box 220 93 230 118)
     :tabstop         nil
     :groupstart      nil
     :buddy-widget    :blue-sel-color-text
     :buddy-side      :right
     :delayed         nil
     :orientation     :vertical
     :wrapping        nil
     :use-arrow-keys  t
     :increment       nil
     :range           (list 0 255))
   ;; The apply button.
   (add-icon-edit-tab-widget
     :widget       'button
     :title        "~A p p l y"
     :box          (make-box 20 125 220 145)
     :3d-border    t
     :tabstop      t
     :groupstart   t
     :set-value-fn #'apply-color-values)))


;;;-------------------------------------------------------------------------;;;
;;;  C O L O R S  page initializer / finalizer.                             ;;;
;;;-------------------------------------------------------------------------;;;

;;; Setting up the :colors tab page items and values.
(defmethod ICON-EDIT-SETUP-PAGE ((icon icon) (id (eql :colors))
                                 &optional socket)
  (let ((fgcolor  (icon-fg-color  icon))
        (bgcolor  (icon-bg-color  icon))
        (selcolor (icon-sel-color icon))
        (*called-programmatically* t))
    (set-dialog-item-value *color-fore-red*   (rgb-red   fgcolor))
    (set-dialog-item-value *color-fore-green* (rgb-green fgcolor))
    (set-dialog-item-value *color-fore-blue*  (rgb-blue  fgcolor))
    (set-dialog-item-value *color-box-fore*   fgcolor)
    (set-dialog-item-value *color-back-red*   (rgb-red   bgcolor))
    (set-dialog-item-value *color-back-green* (rgb-green bgcolor))
    (set-dialog-item-value *color-back-blue*  (rgb-blue  bgcolor))
    (set-dialog-item-value *color-box-back*   bgcolor)
    (set-dialog-item-value *color-sel-red*    (rgb-red   selcolor))
    (set-dialog-item-value *color-sel-green*  (rgb-green selcolor))
    (set-dialog-item-value *color-sel-blue*   (rgb-blue  selcolor))
    (set-dialog-item-value *color-box-sel*    selcolor)))

;;; Done with the :colors tab page.
(defmethod ICON-EDIT-DONE-PAGE ((icon icon) (id (eql :colors)) result)
  (when result
    (apply-color-values)))


;;;-------------------------------------------------------------------------;;;
;;;  C O L O R S  message handlesr                                          ;;;
;;;-------------------------------------------------------------------------;;;

;;; General color-box update.
(defun COLOR-UPDATE (color-box rval gval bval)
  (let ((*called-programmatically* t))
    (set-dialog-item-value
     color-box (make-rgb :red rval :green gval :blue bval))))

(defun APPLY-COLOR-VALUES (&rest args)
  (setf (icon-fg-color  *icon-edit-icon*) (dialog-item-value *color-box-fore*))
  (setf (icon-bg-color  *icon-edit-icon*) (dialog-item-value *color-box-back*))
  (setf (icon-sel-color *icon-edit-icon*) (dialog-item-value *color-box-sel*))
  (icon-reshow *icon-edit-icon*)
  (do-redraws (icon-doc *icon-edit-icon*))
  nil)

;;; Lisp-text / color-box message handlers, there are too many that look the
;;; same, but its not important enough to generalize.

;;; Foreground handlers.

(defun COLOR-BOX-FORE-SET (item new old)
  (unless *called-programmatically*
    (let ((*called-programmatically* t))
      (set-dialog-item-value *color-fore-red*   (rgb-red   new))
      (set-dialog-item-value *color-fore-green* (rgb-green new))
      (set-dialog-item-value *color-fore-blue*  (rgb-blue  new))))
  t)

(defun COLOR-FORE-RED-SET (item new old)
  (cond (*called-programmatically* t)
        ((and (integerp new) (<= 0 new 255))
         (color-update *color-box-fore*
                       new
                       (dialog-item-value *color-fore-green*)
                       (dialog-item-value *color-fore-blue*))
         t)
        (t nil)))

(defun COLOR-FORE-GREEN-SET (item new old)
  (cond (*called-programmatically* t)
        ((and (integerp new) (<= 0 new 255))
         (color-update *color-box-fore*
                       (dialog-item-value *color-fore-red*)
                       new
                       (dialog-item-value *color-fore-blue*))
         t)
        (t nil)))

(defun COLOR-FORE-BLUE-SET (item new old)
  (cond (*called-programmatically* t)
        ((and (integerp new) (<= 0 new 255))
         (color-update *color-box-fore*
                       (dialog-item-value *color-fore-red*)
                       (dialog-item-value *color-fore-green*)
                       new)
         t)
        (t nil)))

;;; Background handlers.

(defun COLOR-BOX-BACK-SET (item new old)
  (unless *called-programmatically*
    (let ((*called-programmatically* t))
      (set-dialog-item-value *color-back-red*   (rgb-red   new))
      (set-dialog-item-value *color-back-green* (rgb-green new))
      (set-dialog-item-value *color-back-blue*  (rgb-blue  new))))
  t)

(defun COLOR-BACK-RED-SET (item new old)
  (cond (*called-programmatically* t)
        ((and (integerp new) (<= 0 new 255))
         (color-update *color-box-back*
                       new
                       (dialog-item-value *color-back-green*)
                       (dialog-item-value *color-back-blue*))
         t)
        (t nil)))

(defun COLOR-BACK-GREEN-SET (item new old)
  (cond (*called-programmatically* t)
        ((and (integerp new) (<= 0 new 255))
         (color-update *color-box-back*
                       (dialog-item-value *color-back-red*)
                       new
                       (dialog-item-value *color-back-blue*))
         t)
        (t nil)))

(defun COLOR-BACK-BLUE-SET (item new old)
  (cond (*called-programmatically* t)
        ((and (integerp new) (<= 0 new 255))
         (color-update *color-box-back*
                       (dialog-item-value *color-back-red*)
                       (dialog-item-value *color-back-green*)
                       new)
         t)
        (t nil)))

;;; Selection handlers.

(defun COLOR-BOX-SEL-SET (item new old)
  (unless *called-programmatically*
    (let ((*called-programmatically* t))
      (set-dialog-item-value *color-sel-red*   (rgb-red   new))
      (set-dialog-item-value *color-sel-green* (rgb-green new))
      (set-dialog-item-value *color-sel-blue*  (rgb-blue  new))))
  t)

(defun COLOR-SEL-RED-SET (item new old)
  (cond (*called-programmatically* t)
        ((and (integerp new) (<= 0 new 255))
         (color-update *color-box-sel*
                       new
                       (dialog-item-value *color-sel-green*)
                       (dialog-item-value *color-sel-blue*))
         t)
        (t nil)))

(defun COLOR-SEL-GREEN-SET (item new old)
  (cond (*called-programmatically* t)
        ((and (integerp new) (<= 0 new 255))
         (color-update *color-box-sel*
                       (dialog-item-value *color-sel-red*)
                       new
                       (dialog-item-value *color-sel-blue*))
         t)
        (t nil)))

(defun COLOR-SEL-BLUE-SET (item new old)
  (cond (*called-programmatically* t)
        ((and (integerp new) (<= 0 new 255))
         (color-update *color-box-sel*
                       (dialog-item-value *color-sel-red*)
                       (dialog-item-value *color-sel-green*)
                       new)
         t)
        (t nil)))


;;;-------------------------------------------------------------------------;;;
;;;  D O N E !                                                              ;;;
;;;-------------------------------------------------------------------------;;;

(provide :ICONEDIT)

